-------------------------------------------------------------------------------
--! @file       AES_EncDec_Datapath.vhd
--! @brief      A controller for AES_EncDec.vhd
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.AES_pkg.all;

entity AES_EncDec_Control is
    generic (
        G_RNDS          : integer := AES_ROUNDS);
    port(
        clk             : in  std_logic;
        rst             : in  std_logic;

        --! Internal
        sel_decrypt     : out std_logic;
        invround        : out std_logic_vector(3 downto 0);
        round           : out std_logic_vector(3 downto 0);
        en_rkey         : out std_logic;
        wr_rkey         : out std_logic;
        en_fkey         : out std_logic;
        en_lkey         : out std_logic;
        sel_fkey        : out std_logic;
        sel_round       : out std_logic;
        sel_in          : out std_logic_vector(1 downto 0);
        en_in           : out std_logic;

        --! External
        init            : in  std_logic;
        done_init       : out std_logic;
        start           : in  std_logic;
        decrypt         : in  std_logic;
        ready           : out std_logic;
        done            : out std_logic);
end AES_EncDec_Control;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of AES_EncDec_Control
-------------------------------------------------------------------------------

architecture behav of AES_EncDec_Control is
    type t_state is (S_RESET, S_WAIT_START, S_INIT_KEY, S_PROCESS);
    signal state        : t_state;
    signal round_reg    : std_logic_vector(3 downto 0);
    signal invround_reg : std_logic_vector(3 downto 0);
    signal done_s       : std_logic;
    signal done_init_s  : std_logic;
    signal decrypt_reg  : std_logic;
begin
    p_fsm: process(clk)
    begin
        if rising_edge(clk) then
            if (rst = '1') then
                state <= S_RESET;
            else
                if (state = S_RESET) then
                    round_reg       <= std_logic_vector(to_unsigned(1,4));
                    invround_reg    <= std_logic_vector(to_unsigned(G_RNDS-1,4));
                    state           <= S_WAIT_START;
                elsif (state = S_WAIT_START) then
                    if (init = '1') then
                        round_reg    <= (others => '0');
                        state        <= S_INIT_KEY;
                    elsif (start = '1') then
                        round_reg    <= round_reg + 1;
                        invround_reg <= invround_reg - 1;
                        state        <= S_PROCESS;
                        decrypt_reg  <= decrypt;
                    end if;
                elsif (state = S_INIT_KEY) then
                    if (round_reg = G_RNDS) then
                        round_reg    <= std_logic_vector(to_unsigned(1,4));
                        state        <= S_WAIT_START;
                    else
                        round_reg    <= round_reg + 1;
                    end if;
                elsif (state = S_PROCESS) then
                    if (round_reg = G_RNDS) then
                        round_reg    <= std_logic_vector(to_unsigned(1,4));
                        invround_reg <= std_logic_vector(to_unsigned(G_RNDS-1,4));
                        state        <= S_WAIT_START;
                    else
                        invround_reg <= invround_reg - 1;
                        round_reg    <= round_reg + 1;
                    end if;
                end if;
                done_init <= done_init_s;
                done      <= done_s;
            end if;
        end if;
    end process;
    round          <= round_reg;
    invround       <= invround_reg;
    sel_decrypt    <= decrypt_reg;

    p_comb: process(start, init, decrypt_reg, decrypt, state, round_reg)
    begin
        --! Default values
        wr_rkey        <= '0';
        en_fkey        <= '0';
        en_lkey        <= '0';
        sel_round      <= '0';
        sel_in         <= "00";
        en_in          <= '0';
        done_s         <= '0';
        done_init_s    <= '0';
        ready          <= '0';
        en_rkey        <= '0';
        sel_fkey       <= '0';

        if (state = S_RESET) then
        elsif (state = S_WAIT_START) then
            ready <= '1';
            if (init = '1') then
                en_lkey <= '1';
                en_fkey <= '1';
            elsif (start = '1') then
                en_in    <= '1';
                en_rkey  <= '1';
                sel_in   <= "10";                
                if (decrypt = '1') then
                    sel_round <= '1';  
                else
                    sel_fkey  <= '1';                    
                end if;            
            end if;
        elsif (state = S_INIT_KEY) then
            if (round_reg = 0) then
                sel_fkey <= '1';
            elsif (round_reg = G_RNDS) then
                done_init_s <= '1';
            end if;
            if (round_reg /= G_RNDS) then
                en_lkey     <= '1';
            end if;            
            wr_rkey     <= '1';
        elsif (state = S_PROCESS) then
            if (round_reg = G_RNDS) then
                done_s  <= '1';
            end if;
            if (decrypt_reg = '1') then
                sel_round <= '1';
                sel_in    <= "01";
            end if;
            en_rkey  <= '1';
            en_in    <= '1';
        end if;
    end process;
end behav;