-------------------------------------------------------------------------------
--! @file       AES_Enc_Control.vhd
--! @brief      A controller for AES_Enc_Control.vhd
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.aes_pkg.all;

entity AES_Enc_Control is
    generic (
        G_RNDS      : integer := AES_ROUNDS);
    port(
        clk         : in  std_logic;
        rst         : in  std_logic;
        start       : in  std_logic;
        init        : in  std_logic;

        sel_fkey    : out std_logic;
        en_fkey     : out std_logic;
        en_rkey     : out std_logic;
        wr_rkey     : out std_logic;
        round       : out std_logic_vector(3 downto 0);
        sel_in      : out std_logic;
        en_in       : out std_logic;
        ready       : out std_logic;
        done        : out std_logic;
        done_init   : out std_logic);
end AES_Enc_Control;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of AES_Enc_Control
-------------------------------------------------------------------------------

architecture behav of AES_Enc_Control is
    type t_state is (S_RESET, S_WAIT_START, S_INIT, S_PROCESS);
    signal state       : t_state;
    signal round_reg   : std_logic_vector(3 downto 0);
    signal done_s      : std_logic;
    signal done_init_s : std_logic;
begin

    p_fsm: process(clk)
    begin
        if rising_edge(clk) then
            if (rst = '1') then
                state <= S_RESET;
            else
                if (state = S_RESET) then
                    round_reg   <= (others => '0');
                    state       <= S_WAIT_START;
                elsif (state = S_WAIT_START) then
                    if (init = '1') then
                        state       <= S_INIT;
                    elsif (start = '1') then
                        round_reg   <= round_reg + 1;
                        state       <= S_PROCESS;
                    end if;
                elsif (state = S_INIT) then
                    if (round_reg = G_RNDS-1) then
                        round_reg <= (others => '0');
                        state     <= S_WAIT_START;
                    else
                        round_reg <= round_reg + 1;
                    end if;
                elsif (state = S_PROCESS) then
                    if (round_reg = G_RNDS-1) then
                        round_reg <= (others => '0');
                        state     <= S_WAIT_START;
                    else
                        round_reg <= round_reg + 1;
                    end if;
                end if;
            end if;
            done        <= done_s;
            done_init   <= done_init_s;
        end if;
    end process;
    round <= round_reg;

    p_comb: process(start, state, round_reg, init)
    begin
        --! Default values
        sel_in      <= '0';
        en_in       <= '0';
        ready       <= '0';
        done_s      <= '0';
        done_init_s <= '0';
        sel_fkey    <= '0';
        en_fkey     <= '0';
        wr_rkey     <= '0';
        en_rkey     <= '0';

        if (state = S_RESET) then
        elsif (state = S_WAIT_START) then
            ready <= '1';
            if (init = '1') then
                en_fkey <= '1';
            elsif (start = '1') then
                sel_in  <= '1';
                en_in   <= '1';
                en_rkey <= '1';            
            end if;
        elsif (state = S_INIT) then
            wr_rkey  <= '1';
            if (round_reg = 0) then
                sel_fkey <= '1';
            elsif (round_reg = G_RNDS-1) then
                done_init_s <= '1';
            end if;
        elsif (state = S_PROCESS) then
            en_in   <= '1';
            en_rkey <= '1';
            if (round_reg = G_RNDS-1) then
                done_s   <= '1';
            end if;
        end if;
    end process;

end behav;