-------------------------------------------------------------------------------
--! @file       AES_Enc_KOF_Control.vhd
--! @brief      A controller for AES_EncDec_KOF.vhd
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

use work.aes_pkg.all;

entity AES_Enc_KOF_Control is
    generic (
        G_RNDS      : integer := AES_ROUNDS);
    port(
        clk         : in  std_logic;
        rst         : in  std_logic;
        start       : in  std_logic;

        round       : out std_logic_vector(3 downto 0);
        sel_in      : out std_logic;
        en_in       : out std_logic;
        ready       : out std_logic;
        done        : out std_logic);
end AES_Enc_KOF_Control;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of AES_Enc_KOF_Control
-------------------------------------------------------------------------------

architecture behav of AES_Enc_KOF_Control is
    type t_state is (S_RESET, S_WAIT_START, S_PROCESS, S_DONE);
    signal state     : t_state;
    signal round_reg : std_logic_vector(3 downto 0);
begin

    p_fsm: process(clk)
    begin
        if rising_edge(clk) then
            if (rst = '1') then
                state <= S_RESET;
            else
                if (state = S_RESET) then
                    round_reg   <= (others => '0');
                    state       <= S_WAIT_START;
                elsif (state = S_WAIT_START) then
                    if (start = '1') then
                        round_reg   <= round_reg + 1;
                        state       <= S_PROCESS;
                    end if;
                elsif (state = S_PROCESS) then
                    if (round_reg = G_RNDS-1) then
                        round_reg   <= (others => '0');
                        state       <= S_DONE;
                    else
                        round_reg   <= round_reg + 1;
                    end if;
                elsif (state = S_DONE) then
                    if (start = '1') then
                        round_reg   <= round_reg + 1;
                        state       <= S_PROCESS;
                    else
                        state       <= S_WAIT_START;
                    end if;
                end if;
            end if;
        end if;
    end process;
    round <= round_reg;

    p_comb: process(start, state, round_reg)
    begin
        --! Default values
        sel_in  <= '0';
        en_in   <= '0';
        ready   <= '0';
        done    <= '0';

        if (state = S_RESET) then
        elsif (state = S_WAIT_START) then
            ready <= '1';
            if start = '1' then
                sel_in <= '1';
                en_in  <= '1';
            end if;
        elsif (state = S_PROCESS) then
            en_in <= '1';
        elsif (state = S_DONE) then
            done  <= '1';
            ready <= '1';
            if start = '1' then
                sel_in <= '1';
                en_in  <= '1';
            end if;
        end if;
    end process;

end behav;